@@ -5,7 +5,7 @@
 set -o errexit
 
 # This is the master location at which to change version number
-VERSION="0.4"
+VERSION="0.5.2"
 
 # Run the script to change the version elsewhere
 perl -pi -e 's/^(release|version) = "([\d.]+)"/$1 = "'$VERSION'"/' docs/source/conf.py
@@ -16,7 +16,8 @@ perl -pi -e 's/^version = "([\d.]+)"/version = "'$VERSION'"/' pyproject.toml
 
 # Build distribution
 rm -rf dist
-pip install build twine
+uv pip install build twine
 python -m build
 echo "Enter PyPI API Token"
+cat ../pypi-api-token-float
 echo __token__ | twine upload --repository pypi dist/* --verbose
