@@ -19,12 +19,13 @@ def rnd_scalar(
 def rnd_array(
     fi: FormatInfo, v: float, mode: RoundMode = RoundMode.TiesToEven, sat: bool = False
 ) -> float:
-    return round_ndarray(fi, np.array([v]), mode, sat).item()
+    a = round_ndarray(fi, np.asarray([v]), mode, sat)
+    return float(a[0])
 
 
 @pytest.mark.parametrize("round_float", (rnd_scalar, rnd_array))
 def test_round_p3109(round_float: Callable) -> None:
-    fi = format_info_p3109(4)
+    fi = format_info_p3109(8, 4)
     assert round_float(fi, 0.0068359375) == 0.0068359375
     assert round_float(fi, 0.0029296875) == 0.0029296875
     assert round_float(fi, 0.0078125) == 0.0078125
@@ -49,7 +50,7 @@ def test_round_p3109(round_float: Callable) -> None:
     assert round_float(fi, 232.1) == np.inf
 
 
-p4min = 2**-10  # smallest subnormal in p4
+p4min = 2**-10  # smallest subnormal in 8p4
 
 
 @pytest.mark.parametrize(
@@ -150,7 +151,7 @@ p4min = 2**-10  # smallest subnormal in p4
 )
 @pytest.mark.parametrize("round_float", (rnd_scalar, rnd_array))
 def test_round_p3109b(round_float: Callable, mode: RoundMode, vals: list) -> None:
-    fi = format_info_p3109(4)
+    fi = format_info_p3109(8, 4)
 
     for sat in (True, False):
         for val, expected in vals:
@@ -302,7 +303,7 @@ p4maxhalfup = (p4max + p4maxup) / 2
 def test_round_p3109_sat(
     round_float: Callable, modesat: tuple[RoundMode, bool], vals: list
 ) -> None:
-    fi = format_info_p3109(4)
+    fi = format_info_p3109(8, 4)
 
     for val, expected in vals:
         assert round_float(fi, val, *modesat) == expected
@@ -382,7 +383,7 @@ some_positive_codepoints = (
     [
         format_info_ocp_e5m2,
         format_info_ocp_e4m3,
-        *p3109_formats,
+        *p3109_binary8_formats,
     ],
 )
 def test_round(fi: FormatInfo) -> None:
@@ -472,7 +473,7 @@ def test_round_ints(round_float: Callable, fi: FormatInfo, mldtype: Type) -> Non
         np.testing.assert_equal(val, mlval)
 
 
-@pytest.mark.parametrize("fi", all_formats)
+@pytest.mark.parametrize("fi", sample_formats)
 @pytest.mark.parametrize("round_float", (rnd_scalar, rnd_array))
 def test_round_roundtrip(round_float: Callable, fi: FormatInfo) -> None:
     if fi.bits <= 8:
@@ -535,18 +536,27 @@ def test_stochastic_rounding(
         count_v1 = np.sum(rs == v1)
 
     print(f"SRBits={srnumbits}, observed = {count_v1}, expected = {expected_up_count} ")
-    # e.g. if expected is 1250/10000, want to be within 0.5,1.5
+    # e.g. if expected is 1250/10000, want to be within 0.75,1.25
     # this is loose, but should still catch logic errors
     atol = n * 2.0 ** (-1 - srnumbits)
-    np.testing.assert_allclose(count_v1, expected_up_count, atol=atol)
+    np.testing.assert_allclose(count_v1, expected_up_count, atol=atol / 2)
 
 
 @pytest.mark.parametrize(
     "rnd",
-    (RoundMode.Stochastic, RoundMode.StochasticFast, RoundMode.StochasticFastest),
+    (
+        RoundMode.Stochastic,
+        RoundMode.StochasticOdd,
+        RoundMode.StochasticFast,
+        RoundMode.StochasticFastest,
+    ),
 )
-def test_stochastic_rounding_scalar_eq_array(rnd: RoundMode) -> None:
-    fi = format_info_p3109(3)
+@pytest.mark.parametrize("srnumbits", [3, 8, 9, 16, 32])
+@pytest.mark.parametrize("sat", (True, False))
+def test_stochastic_rounding_scalar_eq_array(
+    rnd: RoundMode, srnumbits: int, sat: bool
+) -> None:
+    fi = format_info_ocp_e5m2
 
     v0 = decode_ndarray(fi, np.arange(255))
     v1 = decode_ndarray(fi, np.arange(255) + 1)
@@ -554,32 +564,56 @@ def test_stochastic_rounding_scalar_eq_array(rnd: RoundMode) -> None:
     v0 = v0[ok]
     v1 = v1[ok]
 
-    srnumbits = 3
-    for srbits in range(2**srnumbits):
-        for alpha in (0, 0.3, 0.5, 0.6, 0.9, 1.25):
-            v = _linterp(v0, v1, alpha)
-            assert np.isfinite(v).all()
-            val_array = round_ndarray(
+    for alpha in (0, 0.3, 0.5, 0.6, 0.7, 0.9, 1.25):
+        v = _linterp(v0, v1, alpha)
+        assert np.isfinite(v).all()
+        srbits = np.random.randint(0, 2**srnumbits, v.shape)
+
+        val_array = round_ndarray(
+            fi,
+            v,
+            rnd,
+            sat=sat,
+            srbits=srbits,
+            srnumbits=srnumbits,
+        )
+
+        val_scalar = [
+            round_float(
                 fi,
-                v,
+                vi,
                 rnd,
-                sat=False,
-                srbits=np.asarray(srbits),
+                sat=sat,
+                srbits=srbitsi,
                 srnumbits=srnumbits,
             )
+            for vi, srbitsi in zip(v, srbits)
+        ]
 
-            val_scalar = [
-                round_float(
-                    fi,
-                    v,
-                    rnd,
-                    sat=False,
-                    srbits=srbits,
-                    srnumbits=srnumbits,
-                )
-                for v in v
-            ]
-            if alpha < 1.0:
-                assert ((val_array == v0) | (val_array == v1)).all()
+        np.testing.assert_equal(val_array, val_scalar)
 
-            np.testing.assert_equal(val_array, val_scalar)
+        # Ensure faithful rounding
+        if alpha < 1.0:
+            assert ((val_array == v0) | (val_array == v1)).all()
+
+
+def test_large_bfloat() -> None:
+    # from https://github.com/graphcore-research/gfloat/pull/49
+
+    a = 6.6461399789245764e35
+    b = 6.620178494631905e35
+
+    assert b < a
+    rounded_a = round_float(format_info_bfloat16, a, RoundMode.TowardZero)
+    rounded_b = round_float(format_info_bfloat16, b, RoundMode.TowardZero)
+
+    assert rounded_b <= rounded_a
+
+    rounded_a_array = round_ndarray(
+        format_info_bfloat16, np.array([a]), RoundMode.TowardZero
+    )
+    rounded_b_array = round_ndarray(
+        format_info_bfloat16, np.array([b]), RoundMode.TowardZero
+    )
+
+    assert all(rounded_b_array <= rounded_a_array)
